@@ -1540,7 +1540,13 @@ rb_s_gpgme_op_verify_result (VALUE dummy, VALUE vctx)
 	   notation = notation->next)
 	{
 	  VALUE vnotation = rb_class_new_instance(0, NULL, cSigNotation);
-	  rb_iv_set (vnotation, "@name", rb_str_new2 (notation->name));
+	  /* The docs say:
+	   * The name of the notation field. If this is NULL, then the member
+	   * value will contain a policy URL. */
+	  if (notation->name == NULL)
+	    rb_iv_set (vnotation, "@name", Qnil);
+	  else
+	    rb_iv_set (vnotation, "@name", rb_str_new2 (notation->name));
 	  rb_iv_set (vnotation, "@value", rb_str_new2 (notation->value));
 	  rb_ary_push (vnotations, vnotation);
 	}
